#===========================================================================
# KoalaRainbow - a visualization system
# Copyright (C) 2004, Andrew Sutherland (sombrero@alum.mit.edu)
#
# KoalaRainbow is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#===========================================================================
package OnlyTheWind::KoalaRainbow::ConstraintManager;
use strict;

use OnlyTheWind::KoalaRainbow::GlobalSetRegistry;

sub new
{
    my $class = shift;
    my $self  =
    {
        GLOBAL_SET_NAME => '',
        TERMS => {},
        ARGUMENTS => {},
    };
    
    bless ($self, $class);
    return $self;
}

sub globalSetName
{
    my $self = shift;
    
    return $self->{GLOBAL_SET_NAME};
}

sub terms
{
    my $self = shift;
    
    return $self->{TERMS};
}

sub arguments
{
    my $self = shift;
    
    return $self->{ARGUMENTS};
}

# should be in a util class or in the nodes via subclassing or otherwise
# okay, now it's even more of a hack because I forgot that the edge actually
# happens within a querycontext node, which actually makes all kinds of sense,
# but coding when sleepy and watching tv apparently has its downsides.
sub isEdgeNode
{
    my $self = shift;
    my $node = shift;
    
    if(ref($node) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::ContextVariable')
    {
        # verify it's legal for our purposes
        return OnlyTheWind::KoalaRainbow::GlobalSetRegistry->isAllowedEdge(
            $self->globalSetName, $node->name);
    }
    elsif(ref($node) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::QueryContext' &&
          $node->isOnlyEvalNode() &&
          ref($node->evalNode) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::ContextVariable')
    {
        return OnlyTheWind::KoalaRainbow::GlobalSetRegistry->isAllowedEdge(
            $self->globalSetName, $node->evalNode->name);
    }
    else
    {
        return 0;
    }
}

sub isQueryContextNode
{
    my $self = shift;
    my $node = shift;
    
    return ref($node) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::QueryContext';
}

# should be in a util class or in the nodes via subclassing or otherwise
sub isFuncNode
{
    my $self = shift;
    my ($node) = @_;
 
    if(ref($node) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::FunctionCall')
    {
        return 1;
    }
    elsif(ref($node) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::QueryContext' &&
          $node->isOnlyEvalNode() &&
          ref($node->evalNode) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::FunctionCall')
    {
        return 1;
    }
    else
    {
        return 0;
    }           
}

sub addComparisonConstraint
{
    my $self = shift;
    my ($edgeNode, $cmpOp, $val) = @_;
    
    my $terms = $self->{TERMS};
    my $arguments = $self->{ARGUMENTS};
    
    my $edgeName;
    if(ref($edgeNode) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::ContextVariable')
    {
        $edgeName = $edgeNode->name;
    }
    elsif(ref($edgeNode) eq
            'OnlyTheWind::KoalaRainbow::QueryParseNodes::QueryContext')
    {
        $edgeName = $edgeNode->evalNode->name;
    }
    
    my $curStart = undef;
    my $curEnd   = undef;
    
    if(defined($terms->{$edgeName}))
    {
        if(ref($terms->{$edgeName}) eq 'ARRAY')
        {
            ($curStart, $curEnd) = $terms->{$edgeName};
        }
    }
    
    my $rangeColumns;
    if(defined($arguments->{range}))
    {
        $rangeColumns = $arguments->{range};
    }
    else
    {
        $rangeColumns = {};
        $arguments->{range} = $rangeColumns;
    }
    
    if($cmpOp eq '>')
    {
        $terms->{$edgeName} = [$curStart, $val];
        $rangeColumns->{$edgeName} = 1;
    }
    elsif($cmpOp eq '<')
    {
        $terms->{$edgeName} = [$val, $curEnd];
        $rangeColumns->{$edgeName} = 1;
    }
    elsif($cmpOp eq '=')
    {
        $terms->{$edgeName} = $val;
    }
    else
    {
        return 0;
    }
    
    return 1;
}

sub toString
{
    my $self = shift;

    my $str = '[[constraints terms: (';
    while (my ($key, $value) = each %{$self->{TERMS}})
    {
        $str .= $key . ': ';
        if(ref($value) eq 'ARRAY')
        {
            $str .= '[' . $value->[0] . ',' . $value->[1] . ']';
        }
        else
        {
            $str .= $value;
        }
        $str .= ' ';
    }
    $str .= ') arguments: ';
    while (my ($key, $value) = each %{$self->{ARGUMENTS}})
    {
        $str .= $key . ' ';
        if(ref($value) eq 'HASH')
        {
            $str .= '{';
            while (my ($subkey, $subvalue) = each %{$value})
            {
                $str .= $subkey . ': ' . $subvalue . ' ';
            }
            
            $str .= '}';
        }
        else
        {
            $str .= $value;
        }
        $str .= ' ';
    }
    $str .= ']]';
    
    return $str;
}

1;