#===========================================================================
# KoalaRainbow - a visualization system
# Copyright (C) 2004, Andrew Sutherland (sombrero@alum.mit.edu)
#
# KoalaRainbow is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#===========================================================================
package OnlyTheWind::KoalaRainbow::DrawNodes::DrawNode;
use strict;

sub new
{
    my $class = shift;
    my $self  =
    {
        X => 0,
        Y => 0,
        WIDTH => 0,
        HEIGHT => 0,
        FILL_COLOR => undef,
        EDGE_COLOR => undef,
        LAYER => 0,
        START_ANG => undef,
        END_ANG => undef,
        URL => undef,
    };
    
    bless ($self, $class);
    return $self;
}

sub center
{
    my $self = shift;
    
    if(scalar(@_) >= 2)
    {
        my ($cx, $cy) = @_;
        
        $self->{X} = $cx - $self->{WIDTH}/2;
        $self->{Y} = $cy - $self->{HEIGHT}/2;
    }
    
    return ($self->{X} + $self->{WIDTH}/2,
            $self->{Y} + $self->{HEIGHT}/2);
}

sub centerX
{
    my $self = shift;
    
    if(@_)
    {
        my ($cx) = @_;
        
        $self->{X} = $cx - ($self->{WIDTH}/2);
    }
    
    return ($self->{X} + $self->{WIDTH}/2);
}

sub centerY
{
    my $self = shift;
    
    if(@_)
    {
        my ($cy) = @_;

        $self->{Y} = $cy - ($self->{HEIGHT}/2);
    }
    
    return ($self->{Y} + $self->{HEIGHT}/2);
}

sub bounds
{
    my $self = shift;
    
    if(scalar(@_) >= 4)
    {
        my ($left, $top, $right, $bottom) = @_;
        
        $self->{X} = $left;
        $self->{Y} = $top;
        $self->{WIDTH} = $right - $left;
        $self->{HEIGHT} = $bottom - $top;
    }
    
    return ($self->{X}, $self->{Y},
            $self->{X} + $self->{WIDTH},
            $self->{Y} + $self->{HEIGHT});
}

sub upperLeft
{
    my $self = shift;
    
    if(scalar(@_) >= 2)
    {
        my ($left, $top) = @_;
        
        $self->{X} = $left;
        $self->{Y} = $top;
    }
    
    return ($self->{X}, $self->{Y});
}

sub bottomRight
{
    my $self = shift;
    
    if(scalar(@_) >= 2)
    {
        my ($right, $bottom) = @_;

        $self->{WIDTH} = $right - $self->{X};
        $self->{HEIGHT} = $bottom - $self->{Y};;
    }
    
    return ($self->{X} + $self->{WIDTH},
            $self->{Y} + $self->{HEIGHT});
}

sub right
{
    my $self = shift;
    
    if(@_)
    {
        my ($right) = @_;

        $self->{WIDTH} = $right - $self->{X};
    }
    
    return $self->{X} + $self->{WIDTH};
}

sub bottom
{
    my $self = shift;
    
    if(@_)
    {
        my ($bottom) = @_;

        $self->{HEIGHT} = $bottom - $self->{Y};;
    }
    
    return $self->{Y} + $self->{HEIGHT};
}

sub width
{
    my $self = shift;
    
    if(@_)
    {
        $self->{WIDTH} = shift;
    }
    
    return $self->{WIDTH};
}

sub height
{
    my $self = shift;
    
    if(@_)
    {
        $self->{HEIGHT} = shift;
    }
    
    return $self->{HEIGHT};
}

sub left
{
    my $self = shift;
    
    if(@_)
    {
        $self->{X} = shift;
    }
    
    return $self->{X};
}

sub top
{
    my $self = shift;
    
    if(@_)
    {
        $self->{Y} = shift;
    }
    
    return $self->{Y};
}

sub edgeColor
{
    my $self = shift;
    
    if(@_)
    {
        $self->{EDGE_COLOR} = shift;
    }
    
    return $self->{EDGE_COLOR};
}

sub fillColor
{
    my $self = shift;
    
    if(@_)
    {
        $self->{FILL_COLOR} = shift;
    }
    
    return $self->{FILL_COLOR};
}

sub layer
{
    my $self = shift;
    
    if(@_)
    {
        $self->{LAYER} = shift;
    }
    
    return $self->{LAYER};
}

sub startang
{
    my $self = shift;
    
    if(@_)
    {
        $self->{START_ANG} = shift;
    }
    
    return $self->{START_ANG};
}

sub endang
{
    my $self = shift;
    
    if(@_)
    {
        $self->{END_ANG} = shift;
    }
    
    return $self->{END_ANG};
}

sub url
{
    my $self = shift;
    
    if(@_)
    {
        $self->{URL} = shift;
    }
    
    return $self->{URL};
}

sub drawImageMap
{
    return '';
}

1;