#===========================================================================
# KoalaRainbow - a visualization system
# Copyright (C) 2004, Andrew Sutherland (sombrero@alum.mit.edu)
#
# KoalaRainbow is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#===========================================================================
package OnlyTheWind::KoalaRainbow::DrawNodes::Line;
use strict;

use OnlyTheWind::KoalaRainbow::DrawNodes::DrawNode;
@OnlyTheWind::KoalaRainbow::DrawNodes::Line::ISA =
    qw(OnlyTheWind::KoalaRainbow::DrawNodes::DrawNode);  

sub new
{
    my $class = shift;
    my $self  = $class->SUPER::new();
    
    $self->{X1} = 0;
    $self->{Y1} = 0;
    $self->{X2} = 0;
    $self->{Y2} = 0;
    $self->{WIDTH} = undef;
    
    bless ($self, $class);          # reconsecrate
    return $self;
}  

sub draw
{
    my $self = shift;
    my ($engine) = @_;
    
    $engine->line($self->x1, $self->y1, $self->x2, $self->y2,
                  $self->edgeColor, $self->width);
}

sub bounds
{
    my $self = shift;
    
    my ($x1, $y1, $x2, $y2) =
        ($self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2});
    
    my ($left, $top, $right, $bottom);
    
    if($x1 < $x2)
    {
        $left  = $x1;
        $right = $x2;
    }
    else
    {
        $left  = $x2;
        $right = $x1;
    }
    
    if($y1 < $y2)
    {
        $top = $y1;
        $bottom = $y2;
    }
    else
    {
        $top = $y2;
        $bottom = $y1;
    }
    
    return ($left, $top, $right, $bottom);
}

sub x1
{
    my $self = shift;
    
    if(@_)
    {
        $self->{X1} = shift;
    }
    
    return $self->{X1};
}

sub x2
{
    my $self = shift;
    
    if(@_)
    {
        $self->{X2} = shift;
    }
    
    return $self->{X2};
}

sub y1
{
    my $self = shift;
    
    if(@_)
    {
        $self->{Y1} = shift;
    }
    
    return $self->{Y1};
}

sub y2
{
    my $self = shift;
    
    if(@_)
    {
        $self->{Y2} = shift;
    }
    
    return $self->{Y2};
}

sub width
{
    my $self = shift;
    if(@_)
    {
        $self->{WIDTH} = shift;
    }
    
    return $self->{WIDTH};
}

1;