#===========================================================================
# KoalaRainbow - a visualization system
# Copyright (C) 2004, Andrew Sutherland (sombrero@alum.mit.edu)
#
# KoalaRainbow is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#===========================================================================
package OnlyTheWind::KoalaRainbow::QueryParseNodes::GlobalVariable;
use strict;

use OnlyTheWind::KoalaRainbow::Environment;

sub new
{
    my $class = shift;
    
    my $self  =
    {
        NAME  => undef,
    };
    
    bless ($self, $class);
    return $self;
}

sub clone
{
    return shift;
}

sub name
{
    my $self = shift;
    
    if (@_) 
    {
        $self->{NAME} = shift;
    }
    
    return $self->{NAME};
}

# For the purposes of our query, we are constant (non-edge). 
sub isConstant
{
    return 1;
}

sub toString
{
    my $self = shift;

    if($self->{NAME})
    {
        return '$' . $self->{NAME};
    }
    else
    {
        return 'unhappy global variable';
    }
}

sub propagateConstraints
{
    return 0;
}

sub partialEvaluate
{
    # we can't get any simpler than this.  it's up to whoever consumes us to
    #  properly evaluate us.
    my $self = shift;
    
    return $self;
}

sub evaluate
{
    my $self = shift;
    
    my $varname = $self->{NAME};
    
    if($varname)
    {
        return OnlyTheWind::KoalaRainbow::Environment->getany($varname);
    }
    else
    {
        return undef;
    }
}

sub evaluateSet
{
    my $self = shift;
    
    my $varname = $self->{NAME};
    
    if($varname)
    {
        return OnlyTheWind::KoalaRainbow::Environment->getset($varname);
    }
    else
    {
        return undef;
    }
}

sub evaluateValue
{
    my $self = shift;
    
    my $varname = $self->{NAME};
    
    if($varname)
    {
        return OnlyTheWind::KoalaRainbow::Environment->getval($varname);
    }
    else
    {
        return undef;
    }
}

1;
