#===========================================================================
# KoalaRainbow - a visualization system
# Copyright (C) 2004, Andrew Sutherland (sombrero@alum.mit.edu)
#
# KoalaRainbow is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#===========================================================================
package OnlyTheWind::KoalaRainbow::QueryParseNodes::QueryContext;
use strict;

use OnlyTheWind::KoalaRainbow::ContextManager;
use OnlyTheWind::KoalaRainbow::ConstraintManager;
use OnlyTheWind::KoalaRainbow::GlobalSetRegistry;

sub new
{
    my $class = shift;
    my $self  =
    {
        GLOBAL_SET  => undef,
        EVAL_NODE => undef,
        CONSTRAINT => undef,
        NEXT_CONTEXT_NODE => undef,
        SORT_BY => undef,
        CONSTRAINT_MGR => undef,
    };
    
    bless ($self, $class);
    return $self;
}


sub clone
{
    my $self = shift;
    
    my $newMe = OnlyTheWind::KoalaRainbow::QueryParseNodes::QueryContext->new();

    $newMe->globalSet($self->globalSet());
    if(defined($self->evalNode()))
    {
        $newMe->evalNode($self->evalNode()->clone());
    }
    if(defined($self->constraint()))
    {
        $newMe->constraint($self->constraint()->clone());
    }
    if(defined($self->nextContext()))
    {
        $newMe->nextContext($self->nextContext()->clone());
    }
    # no current sort-by
    # constraint manager added later... I think
    
    return $newMe;
}

sub globalSet
{
    my $self = shift;
    
    if (@_) 
    {
        $self->{GLOBAL_SET} = shift;
    }
    
    return $self->{GLOBAL_SET};   
}

sub constraint
{
    my $self = shift;
    
    if (@_) 
    {
        $self->{CONSTRAINT} = shift;
    }
    
    return $self->{CONSTRAINT};   
}

sub evalNode
{
    my $self = shift;
    
    if (@_) 
    {
        $self->{EVAL_NODE} = shift;
    }
    
    return $self->{EVAL_NODE};   
}

sub isOnlyEvalNode
{
    my $self = shift;

        GLOBAL_SET  => undef,
        EVAL_NODE => undef,
        CONSTRAINT => undef,
        NEXT_CONTEXT_NODE => undef,
        SORT_BY => undef,
        CONSTRAINT_MGR => undef,

    return(defined($self->{EVAL_NODE}) &&
           !defined($self->{GLOBAL_SET}) &&
           !defined($self->{CONSTRAINT}) &&
           !defined($self->{NEXT_CONTEXT_NODE}) &&
           !defined($self->{SORT_BY}));
}

sub nextContext
{
    my $self = shift;
    
    if (@_) 
    {
        $self->{NEXT_CONTEXT_NODE} = shift;
    }
    
    return $self->{NEXT_CONTEXT_NODE};   
}

sub sortBy
{
    my $self = shift;
    
    if (@_) 
    {
        $self->{SORT_BY} = shift;
    }
    
    return $self->{SORT_BY};   
}

sub toString
{
    my $self = shift;

    my $str = '';

    if($self->{GLOBAL_SET})
    {
        $str .= '{Global Set: ' . $self->{GLOBAL_SET} . '}';
    }
    if($self->{CONSTRAINT_MGR})
    {
        $str .= $self->{CONSTRAINT_MGR}->toString();
    }
    if($self->{EVAL_NODE})
    {
        $str .= ' ' . $self->{EVAL_NODE}->toString();
    }
    if($self->{CONSTRAINT})
    {
        $str .= ' [' . $self->{CONSTRAINT}->toString() . ']';
    }
    if($self->{NEXT_CONTEXT})
    {
        $str .= ' ->' . $self->{NEXT_CONTEXT}->toString();
    }
    if($self->{SORT_BY})
    {
        $str .= ' ::' . $self->{SORT_BY};
    }
    
    return $str;
}

sub isConstant
{
    # no way are we constant.
    return 0;
}

sub _trace
{
    my $msg = shift;
    print $msg . "\n";
}

sub propagateConstraints
{
    return 0;
}

sub partialEvaluate
{
    my ($self, $ctxMgr) = @_;

    my $constraintMgr = undef;

    if($self->{GLOBAL_SET})
    {
        $constraintMgr = OnlyTheWind::KoalaRainbow::ConstraintManager->new();
    }
    if($self->{EVAL_NODE})
    {
        $self->{EVAL_NODE} = $self->{EVAL_NODE}->partialEvaluate($ctxMgr);
    }
    if($self->{CONSTRAINT})
    {
        $self->{CONSTRAINT} = $self->{CONSTRAINT}->partialEvaluate($ctxMgr);
        if($constraintMgr)
        {
            $self->{CONSTRAINT}->propagateConstraints($ctxMgr, $constraintMgr);
            $self->{CONSTRAINT_MGR} = $constraintMgr;
        }
    }
    if($self->{NEXT_CONTEXT})
    {
        $self->{NEXT_CONTEXT} = $self->{NEXT_CONTEXT}->partialEvaluate($ctxMgr);
    }
    if($self->{SORT_BY})
    {
        $self->{SORT_BY} = $self->{SORT_BY}->partialEvaluate($ctxMgr);
    }
    
    return $self;
}

sub evaluate
{
    my ($self, $ctxMgr) = @_;
     
    if(!defined($ctxMgr))
    {
        warn 'Received null context manager. Returning undef.';
        return undef;
    }
    
    if($self->{GLOBAL_SET})
    {
        my $loader = OnlyTheWind::KoalaRainbow::GlobalSetRegistry->getSetLoader(
             $self->{GLOBAL_SET});
        my $newCtxMgr = $loader->($self->{CONSTRAINT_MGR});
        $newCtxMgr->parent($ctxMgr);
        $ctxMgr = $newCtxMgr;
    }
    
    if($self->{EVAL_NODE})
    {
        my $nodeVal = $self->{EVAL_NODE}->evaluateSet($ctxMgr);
        
        #my $nextContext = OnlyTheWind::KoalaRainbow::ContextManager->new();
        #$nextContext->nodes([$nodeVal]);
        #$nextContext->parent($ctxMgr);
        
        #$ctxMgr = $nextContext;
        
        # nodeVal should already be a context manager.
        $ctxMgr = $nodeVal;
        
        if(!defined($ctxMgr))
        {
            if(!defined($ctxMgr))
            {
                die "Node '" . $self->{EVAL_NODE}->toString() . 
                      "' evaluated to undef. Returning undef.";
            }        
        }
    }
    
    if($self->{CONSTRAINT})
    {
        my $consNode = $self->{CONSTRAINT};
           
        my $nextContext = OnlyTheWind::KoalaRainbow::ContextManager->new();
        $nextContext->parent($ctxMgr);

        $ctxMgr->loop(sub
        {
            my ($testContext, $node) = @_;
                        
            if($consNode->evaluate($testContext))
            {
                $nextContext->addNode($node);
            }
        });
        
        $ctxMgr = $nextContext;
    }
    
    if($self->{NEXT_CONTEXT_NODE})
    {
        my $nextContextNode = $self->{NEXT_CONTEXT_NODE};
        return $nextContextNode->evaluate($ctxMgr);
    }
    else
    {
        return $ctxMgr;
    }
}

sub evaluateSet
{
    my ($self, $ctxMgr) = @_;
    
    return $self->evaluate($ctxMgr);
}

sub evaluateValue
{
    my ($self, $ctxMgr) = @_;
    
    my $res = $self->evaluate($ctxMgr);
    
    if(ref($res) eq 'OnlyTheWind::KoalaRainbow::ContextManager')
    {
        return $res->nodes()->[0];
    }
    else
    {
        return $res;
    }
}

1;
