#===========================================================================
# KoalaRainbow - a visualization system
# Copyright (C) 2004, Andrew Sutherland (sombrero@alum.mit.edu)
#
# KoalaRainbow is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#===========================================================================
package koalarainbow;

use MT::Template::Context;
use MT::ConfigMgr;
use MT::Plugin;
use MT;
use strict;

use OnlyTheWind::KoalaRainbow::KoalaRainbow;
use OnlyTheWind::KoalaRainbow::Visualization;

my $plugin;

$plugin = new MT::Plugin();
$plugin->name("Koala Rainbow, v." . 
              $OnlyTheWind::KoalaRainbow::KoalaRainbow::VERSION);
$plugin->description(
             "Visualization engine with nifty query support and some tags.");
$plugin->config_link('krgui.cgi');

my $cfgmgr = MT::ConfigMgr->instance;

my $swebpath = $cfgmgr->StaticWebPath;
if(!$swebpath =~ m!/$!)
{
    $swebpath .= '/';
}

$plugin->doc_link($swebpath . 'docs/koalarainbow/');


MT->add_plugin($plugin);

# Register stuff
#MT::Template::Context->add_tag("KRQueryValue", \&KRQueryValue);
#MT::Template::Context->add_container_tag("KRQuerySet", \&KRQuerySet);

MT::Template::Context->add_container_tag("KRVisualization", \&KRVisualization);

MT::Template::Context->add_container_tag("KRVloop", \&KRVloop);
MT::Template::Context->add_container_tag("KRVloop0", \&KRVloop);
MT::Template::Context->add_container_tag("KRVloop1", \&KRVloop);
MT::Template::Context->add_container_tag("KRVloop2", \&KRVloop);
MT::Template::Context->add_container_tag("KRVloop3", \&KRVloop);
MT::Template::Context->add_container_tag("KRVloop4", \&KRVloop);


MT::Template::Context->add_container_tag("KRVif", \&KRVif);
MT::Template::Context->add_container_tag("KRVelse", \&KRVelse);


MT::Template::Context->add_tag("KRVfilter", \&KRVfilter);

MT::Template::Context->add_tag("KRVsetset", \&KRVsetset);
MT::Template::Context->add_tag("KRVsetval", \&KRVsetval);

MT::Template::Context->add_tag("KRVprint", \&KRVprint);

MT::Template::Context->add_tag("KRVsquare", \&shapeRect);
MT::Template::Context->add_tag("KRVrect", \&shapeRect);

MT::Template::Context->add_tag("KRVcircle", \&shapeEllipse);
MT::Template::Context->add_tag("KRVellipse", \&shapeEllipse);


MT::Template::Context->add_tag("KRVline", \&shapeLine);


#sub KRQueryValue
#{
#    my ($context, $args) = @_;
# 
# 	'';
#}

#sub KRQuerySet
#{
#	
#}

sub KRVisualization
{
    my ($context, $args) = @_;
    
    my $vis = OnlyTheWind::KoalaRainbow::Visualization->new();
    $vis->procVisualization($context, $args);

    $context->stash('KRvis', $vis);
    
    # use the builder to evaluate our contents.
    $vis->procBody($context, $args);
    
    if($context->stash('KRsavevisobj'))
    {
        $context->stash('KRsavevisobj', $vis);
    }
    
    $vis->run();
        
    $vis->render($context);
    
    return $vis->getHTMLSnippet($context);
}

sub _gimmeVis
{
    my $context = shift;
    
    return $context->stash('KRvis');
}

sub KRVloop
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    $vis->cmdLoop($context, $args);
    
    '';
}

sub KRVprint
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    return $vis->procPrint($context, $args);
}

sub KRVif
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    return $vis->cmdIf($context, $args);
}

sub KRVelse
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    return $vis->cmdElse($context, $args);
}

sub KRVfilter
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);

    $vis->cmdFilter($args);
    
    '';
}

sub KRVsetset
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);

    $vis->cmdSetSets($args);
    
    '';
}

sub KRVsetval
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);

    $vis->cmdSetVals($args);
}

#################### SHAPES ####################
sub shapeRect
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    $vis->cmdRect($args);
    return '';
}

sub shapeEllipse
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    $vis->cmdEllipse($args);
    return '';
}

sub shapeLine
{
    my ($context, $args) = @_;
    my $vis = _gimmeVis($context);
    
    $vis->cmdLine($args);
    return '';   
}

1;